from plone.app.multilingual import logger
from plone.app.multilingual.subscriber import set_recursive_language
from plone.base.interfaces import ILanguage
from plone.base.utils import unrestricted_construct_instance
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from time import time
from zope.component import getUtility

import transaction


SHARED_NAME = "shared"  # old shared folder name
OLD_PREFIX = "old_"  # temporary prefix while migrating
PROFILE_ID = "profile-plone.app.multilingual:default"


def reimport_css_registry(context):
    setup = getToolByName(context, "portal_setup")
    setup.runImportStepFromProfile(
        "profile-plone.app.multilingual:default",
        "cssregistry",
        run_dependencies=False,
        purge_old=False,
    )

    # Refresh css
    cssregistry = getToolByName(context, "portal_css")
    cssregistry.cookResources()


def migration_pam_1_to_2(context):
    """Migration plone.app.multilingual 1.x to 2.0 by renaming existing
    language folders and creating new LRF containers where existing
    objects are moved into. Old shared content is moved to portal
    root."""

    s1 = time()
    type_name = "LRF"
    ltool = getToolByName(context, "portal_languages")
    utool = getToolByName(context, "portal_url")
    wtool = getToolByName(context, "portal_workflow")
    portal = utool.getPortalObject()

    logger.info("Starting migration of language folders.")

    for code, name in ltool.listSupportedLanguages():
        if code not in portal:
            continue

        older = portal[code]

        if older.portal_type == type_name:
            logger.info(f"'{code}' is already a {type_name}, skipping.")
            continue

        # PHASE 1: rename old language folders
        s2 = time()
        old_id = OLD_PREFIX + older.id
        logger.info(f"{code} - Phase 1: Renaming to '{old_id}' ...")
        portal.manage_renameObject(older.id, old_id)
        logger.info(
            "{} - Phase 1: Renaming to '{}' took {:.2f}s.".format(
                code, old_id, time() - s2
            )
        )
        transaction.savepoint()

        # PHASE 2: move content to new LRF
        s3 = time()
        old = portal[old_id]
        logger.info(f"{code} - Phase 2: Moving objects into new LRF...")

        unrestricted_construct_instance(type_name, portal, code)
        new = portal[code]
        new.setTitle(name)
        ILanguage(new).set_language(code)

        state = wtool.getInfoFor(new, "review_state", None)
        available_transitions = [t["id"] for t in wtool.getTransitionsFor(new)]
        if state != "published" and "publish" in available_transitions:
            wtool.doActionFor(new, "publish")
        new.reindexObject()
        transaction.savepoint()

        new.manage_pasteObjects(old.manage_cutObjects(ids=old.objectIds()))

        logger.info(
            "{} - Phase 2: Moving objects to LRF took in {:.2f}s.".format(
                code, time() - s3
            )
        )

        transaction.savepoint()

        # PHASE 3: remove old language folders
        s4 = time()
        portal.manage_delObjects(
            ids=[
                old_id,
            ]
        )
        logger.info(
            "{} - Phase 3: Removing '{}' took {:.2f}s.".format(
                code, old_id, time() - s4
            )
        )

        transaction.savepoint()

    # PHASE 4: Old shared folder
    if SHARED_NAME in portal:
        s5 = time()
        shared = portal[SHARED_NAME]
        logger.info(f"{SHARED_NAME} - Phase 4: Moving content to root...")

        portal.manage_pasteObjects(shared.manage_cutObjects(ids=shared.objectIds()))

        logger.info(
            "{} - Phase 4: Moving objects into root took {:.2f}s.".format(
                SHARED_NAME, time() - s5
            )
        )

        transaction.savepoint()

        s6 = time()
        portal.manage_delObjects(
            ids=[
                SHARED_NAME,
            ]
        )
        logger.info(f"{SHARED_NAME} - Phase 5: Removing it took {time() - s6:.2f}s.")

    logger.info(f"All finished in {time() - s1}.")


def upgrade_to_3(context):
    registry = getUtility(IRegistry)

    # don't re-create if it already exists
    key = (
        "plone.app.multilingual.interfaces.IMultiLanguageExtraOptionsSchema."
        "bypass_languageindependent_field_permission_check"
    )
    if key in registry:
        return

    context.runImportStepFromProfile(
        PROFILE_ID.replace("default", "to_3"),
        "plone.app.registry",
    )


def upgrade_to_4(context):
    context.runImportStepFromProfile(
        PROFILE_ID.replace("default", "to_4"),
        "plone.app.registry",
    )


def update_old_layouts(context):
    """We may have no longer existing layouts layouts set."""
    DEFAULT = "folder_listing"
    MAPPING = {
        "folder_summary_view": "summary_view",
        "folder_full_view": "full_view",
        "folder_tabular_view": "tabular_view",
        "atct_album_view": "album_view",
    }
    types_tool = getToolByName(context, "portal_types")
    catalog = getToolByName(context, "portal_catalog")
    our_types = ["LIF", "LRF"]
    for type_name in our_types:
        fti = types_tool.get(type_name)
        if fti is None:
            # Should not happen, but I like upgrade steps to be forgiving.
            logger.warning("Could not find portal_type %s.", type_name)
            continue

        # First update the FTI.
        old_view_methods = fti.view_methods
        view_methods = []
        for name in fti.view_methods:
            name = MAPPING.get(name, name)
            view_methods.append(name)
        if DEFAULT not in view_methods:
            view_methods.append(DEFAULT)
        view_methods = tuple(view_methods)
        if old_view_methods != view_methods:
            fti.view_methods = view_methods
            logger.info("Updated old view methods in FTI %s.", type_name)
        if fti.default_view not in view_methods:
            default_view = MAPPING.get(fti.default_view, DEFAULT)
            logger.info("Set default_view of FTI %s to %s.", type_name, default_view)
            fti.default_view = default_view

        # Now update all instances of this FTI.
        for brain in catalog.unrestrictedSearchResults(portal_type=type_name):
            obj = brain.getObject()
            layout = obj.getProperty("layout", None)
            if layout is None or layout in view_methods:
                continue
            new_layout = MAPPING.get(layout)
            if not new_layout:
                # Use the default view: remove the explicit layout.
                obj._delProperty("layout")
                logger.info(
                    "Removed property 'layout' with value %r at %s",
                    layout,
                    "/".join(obj.getPhysicalPath()),
                )
                continue
            obj._updateProperty("layout", new_layout)
            logger.info(
                "Updated property 'layout' to %r at %s",
                new_layout,
                "/".join(obj.getPhysicalPath()),
            )


def fix_indonesian_language(context):
    """Fix the Indonesian language root folder, if it is there.

    Indonesian needs special handling: its language code "id" is not allowed in
    Plone as content id, so its LRF is called "id-id".
    Not all parts of the code were always aware of this.
    Result is that this LRF may have English (or nothing) as language.
    """
    utool = getToolByName(context, "portal_url")
    portal = utool.getPortalObject()
    if "id-id" not in portal.objectIds():
        logger.info("Indonesian folder not available, so no need to fix.")
        return
    folder = portal["id-id"]
    if ILanguage(folder).get_language() == "id":
        logger.info("Indonesian folder 'id-id' already has language 'id'.")
        return
    logger.info("Recursively setting language of 'id-id' folder to Bahasa Indonesia.")
    set_recursive_language(folder, "id")



from AccessControl import getSecurityManager
from Acquisition import aq_parent
from plone.app.multilingual import _
from plone.app.multilingual.browser.interfaces import ITranslateMenu
from plone.app.multilingual.browser.interfaces import ITranslateSubMenuItem
from plone.app.multilingual.browser.utils import is_language_independent
from plone.app.multilingual.browser.vocabularies import translated_languages
from plone.app.multilingual.browser.vocabularies import translated_urls
from plone.app.multilingual.browser.vocabularies import untranslated_languages
from plone.app.multilingual.interfaces import ILanguageRootFolder
from plone.app.multilingual.interfaces import IMultiLanguageExtraOptionsSchema
from plone.app.multilingual.interfaces import IPloneAppMultilingualInstalled
from plone.app.multilingual.interfaces import ITG
from plone.app.multilingual.interfaces import ITranslatable
from plone.app.multilingual.interfaces import LANGUAGE_INDEPENDENT
from plone.app.multilingual.permissions import ManageTranslations
from plone.base.defaultpage import is_default_page
from plone.base.interfaces import ILanguage
from plone.base.utils import safe_text
from plone.memoize import view
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from zope.browsermenu.menu import BrowserMenu
from zope.browsermenu.menu import BrowserSubMenuItem
from zope.component import getUtility
from zope.component.hooks import getSite
from zope.interface import implementer


@implementer(ITranslateMenu)
class TranslateMenu(BrowserMenu):
    def getMenuItems(self, context, request):
        """Return menu item entries in a TAL-friendly form."""
        # Settings
        site_url = getSite().absolute_url()
        language_tool = getToolByName(context, "portal_languages")
        show_flags = language_tool.showFlags
        try:
            lang_names = request.locale.displayNames.languages
        except AttributeError:
            lang_names = {}

        registry = getUtility(IRegistry)
        settings = registry.forInterface(
            IMultiLanguageExtraOptionsSchema, prefix="plone"
        )
        if settings.redirect_babel_view:
            translation_view = "babel_edit"
        else:
            translation_view = "edit"

        # Content
        content = context
        content_url = context.absolute_url()
        content_language = ILanguage(content).get_language()
        content_translatable = not (
            content_language == LANGUAGE_INDEPENDENT
            or is_language_independent(content)
            or ILanguageRootFolder.providedBy(content)
        )
        content_translated = translated_languages(content)
        content_untranslated = untranslated_languages(content)
        content_translation_group = ITG(content, "")
        if not ITranslatable.providedBy(content):
            content = None

        # Folder when content is default page
        folder = aq_parent(context)
        if not is_default_page(folder, context):
            folder = None
        if folder and ITranslatable.providedBy(folder):  # noqa
            folder_url = folder.absolute_url()
            folder_language = ILanguage(folder).get_language()
            folder_translatable = not (
                folder_language == LANGUAGE_INDEPENDENT
                or is_language_independent(folder)
                or ILanguageRootFolder.providedBy(folder)
            )
            folder_translated = translated_languages(folder)
            folder_untranslated = untranslated_languages(folder)
            folder_translation_group = ITG(folder, "")
        else:
            folder_url = ""
            folder_language = ""
            folder_translatable = False
            folder_translated = []
            folder_untranslated = []
            folder_translation_group = ""

        # Assets folder
        assets_folder_url = None
        assets_folder_title = None
        pc = getToolByName(getSite(), "portal_catalog")
        results = pc.unrestrictedSearchResults(
            portal_type="LIF", Language=ILanguage(context).get_language()
        )
        for brain in results:
            assets_folder_url = brain.getURL() + "/folder_contents"
            assets_folder_title = safe_text(brain.Title)
            break
        # Menu items
        results = []
        results_folder = []
        results_content = []

        if folder_translatable:
            # Folder translation view
            lang_name = lang_names.get(folder_language, folder_language)
            results_folder.append(
                {
                    "title": _(
                        "edit_translation",
                        default="Edit ${lang_name}",
                        mapping={"lang_name": lang_name},
                    ),
                    "description": _(
                        "description_babeledit_menu",
                        default="Edit {lang_name} with the two-column translation view",  # noqa
                        mapping={"lang_name": lang_name},
                    ),
                    "action": folder_url + "/" + translation_view,
                    "selected": False,
                    "icon": None,
                    "extra": {
                        "id": "_edit_folder_babel_edit",
                        "separator": None,
                        "class": "",
                    },
                    "submenu": None,
                }
            )

        if (
            folder and folder_untranslated and False
        ):  # disabled in favor of cut & paste  # noqa
            # Set folder language
            results_folder.append(
                {
                    "title": _("title_set_language", default="Change content language"),
                    "description": _(
                        "description_set_language",
                        default="Move the translation under another language folder",  # noqa
                    ),
                    "action": folder_url + "/update_language",
                    "selected": False,
                    "icon": None,
                    "extra": {
                        "id": "_set_folder_language",
                        "separator": None,
                        "class": "",
                    },
                    "submenu": None,
                }
            )

        if folder_translatable:
            for idx, lang in enumerate(folder_untranslated):
                lang_name = lang_names.get(lang.value, lang.title)
                lang_id = lang.value
                icon = (
                    show_flags and language_tool.getFlagForLanguageCode(lang_id) or None
                )  # noqa
                results_folder.append(
                    {
                        "title": _(
                            "create_translation",
                            default="Create ${lang_name}",
                            mapping={"lang_name": lang_name},
                        ),
                        "description": _(
                            "description_translate_into",
                            default="Translate into ${lang_name}",
                            mapping={"lang_name": lang_name},
                        ),
                        "action": "%s/@@create_translation?language=%s"
                        % (folder_url, lang_id),
                        "selected": False,
                        "icon": icon,
                        "width": "14",
                        "height": "11",
                        "extra": {
                            "id": "translate_folder_into_%s" % lang_id,
                            "separator": None,
                            "class": "contentmenuflags",
                        },
                        "submenu": None,
                    }
                )
            urls = translated_urls(folder)
            for lang in folder_translated:
                if lang.value not in urls.by_token:
                    # omit if translation is not permitted to access.
                    continue
                lang_name = lang_names.get(lang.value, lang.title)
                lang_id = lang.value
                icon = (
                    show_flags and language_tool.getFlagForLanguageCode(lang_id) or None
                )  # noqa
                results_folder.append(
                    {
                        "title": _(
                            "edit_translation",
                            default="Edit ${lang_name}",
                            mapping={"lang_name": lang_name},
                        ),
                        "description": _(
                            "description_babeledit_menu",
                            default="Edit {lang_name} with the two-column translation view",  # noqa
                            mapping={"lang_name": lang_name},
                        ),
                        "action": (
                            urls.getTerm(lang_id).title + "/" + translation_view
                        ),
                        "selected": False,
                        "icon": icon,
                        "width": "14",
                        "height": "11",
                        "extra": {
                            "id": "babel_edit_%s" % lang_id,
                            "separator": None,
                            "class": "contentmenuflags",
                        },
                        "submenu": None,
                    }
                )
            # Manage folder translations
            results_folder.append(
                {
                    "title": _(
                        "title_modify_translations", default="Manage translations"
                    ),
                    "description": _(
                        "description_modify_translations",
                        default="Add or delete translations",
                    ),
                    "action": folder_url + "/modify_translations",
                    "selected": False,
                    "icon": None,
                    "extra": {
                        "id": "_modify_folder_translations",
                        "separator": None,
                        "class": "",
                    },
                    "submenu": None,
                }
            )
            # Universal link
            if folder_translation_group:
                results_folder.append(
                    {
                        "title": _("universal_link", default="Universal link"),
                        "description": _(
                            "description_universal_link",
                            default="Universal link to the content in user's preferred language",  # noqa
                        ),
                        "action": "%s/@@multilingual-universal-link/%s"
                        % (site_url, folder_translation_group),
                        "selected": False,
                        "icon": None,
                        "extra": {
                            "id": "_universal_folder_link",
                            "separator": None,
                            "class": "",
                        },
                        "submenu": None,
                    }
                )

        if results_folder:
            # Folder translation separator
            results.append(
                {
                    "title": _(
                        "title_translate_current_folder", default="Folder translation"
                    ),
                    "description": "",
                    "action": None,
                    "selected": False,
                    "icon": None,
                    "extra": {
                        "id": "translateFolderHeader",
                        "separator": "actionSeparator",
                        "class": "plone-toolbar-submenu-header",
                    },
                    "submenu": None,
                }
            )
        results.extend(results_folder)

        lang_name = lang_names.get(content_language, content_language)

        # Content language
        if content_untranslated and False:  # disabled in favor of cut & paste
            results_content.append(
                {
                    "title": _("title_set_language", default="Change content language"),
                    "description": _(
                        "description_set_language",
                        default="Move the translation under another language folder",  # noqa
                    ),
                    "action": content_url + "/update_language",
                    "selected": False,
                    "icon": None,
                    "extra": {"id": "_set_language", "separator": None, "class": ""},
                    "submenu": None,
                }
            )

        if content_translatable:
            # Content translation view
            results_content.append(
                {
                    "title": _(
                        "edit_translation",
                        default="Edit ${lang_name}",
                        mapping={"lang_name": lang_name},
                    ),
                    "description": _(
                        "description_babeledit_menu",
                        default="Edit {lang_name} with the two-column translation view",  # noqa
                        mapping={"lang_name": lang_name},
                    ),
                    "action": content_url + "/" + translation_view,
                    "selected": False,
                    "icon": None,
                    "extra": {
                        "id": "_edit_babel_edit",
                        "separator": None,
                        "class": "",
                    },
                    "submenu": None,
                }
            )

        if content_translatable:
            for idx, lang in enumerate(content_untranslated):
                lang_name = lang_names.get(lang.value, lang.title)
                lang_id = lang.value
                icon = (
                    show_flags and language_tool.getFlagForLanguageCode(lang_id) or None
                )  # noqa
                results_content.append(
                    {
                        "title": _(
                            "create_translation",
                            default="Create ${lang_name}",
                            mapping={"lang_name": lang_name},
                        ),
                        "description": _(
                            "description_translate_into",
                            default="Translate into ${lang_name}",
                            mapping={"lang_name": lang_name},
                        ),
                        "action": "%s/@@create_translation?language=%s"
                        % (content_url, lang_id),
                        "selected": False,
                        "icon": icon,
                        "width": "14",
                        "height": "11",
                        "extra": {
                            "id": "translate_into_%s" % lang_id,
                            "separator": None,
                            "class": "contentmenuflags",
                        },
                        "submenu": None,
                    }
                )
            urls = translated_urls(content)
            for lang in content_translated:
                if lang.value not in urls.by_token:
                    # omit if translation is not permitted to access.
                    continue
                lang_name = lang_names.get(lang.value, lang.title)
                lang_id = lang.value
                icon = (
                    show_flags and language_tool.getFlagForLanguageCode(lang_id) or None
                )  # noqa
                results_content.append(
                    {
                        "title": _(
                            "edit_translation",
                            default="Edit ${lang_name}",
                            mapping={"lang_name": lang_name},
                        ),
                        "description": _(
                            "description_babeledit_menu",
                            default="Edit {lang_name} with the two-column translation view",  # noqa
                            mapping={"lang_name": lang_name},
                        ),
                        "action": (
                            urls.getTerm(lang_id).title + "/" + translation_view
                        ),
                        "selected": False,
                        "icon": icon,
                        "width": "14",
                        "height": "11",
                        "extra": {
                            "id": "babel_edit_%s" % lang_id,
                            "separator": None,
                            "class": "contentmenuflags",
                        },
                        "submenu": None,
                    }
                )
            # Manage content translations
            results_content.append(
                {
                    "title": _(
                        "title_modify_translations", default="Manage translations"
                    ),
                    "description": _(
                        "description_modify_translations",
                        default="Add or delete translations",
                    ),
                    "action": content_url + "/modify_translations",
                    "selected": False,
                    "icon": None,
                    "extra": {
                        "id": "_modify_translations",
                        "separator": None,
                        "class": "",
                    },
                    "submenu": None,
                }
            )
            # Universal link
            if content_translation_group:
                results_content.append(
                    {
                        "title": _("universal_link", default="Universal link"),
                        "description": _(
                            "description_universal_link",
                            default="Universal link to the content in user's preferred language",  # noqa
                        ),
                        "action": "%s/@@multilingual-universal-link/%s"
                        % (site_url, content_translation_group),
                        "selected": False,
                        "icon": None,
                        "extra": {
                            "id": "_universal_link",
                            "separator": None,
                            "class": "",
                        },
                        "submenu": None,
                    }
                )

        if results_folder and results_content:
            # Item translations separator
            results.append(
                {
                    "title": _(
                        "title_translate_current_item", default="Item translation"
                    ),
                    "description": "",
                    "action": None,
                    "selected": False,
                    "icon": None,
                    "extra": {
                        "id": "translateItemHeader",
                        "separator": "actionSeparator",
                        "class": "",
                    },
                    "submenu": None,
                }
            )
        results.extend(results_content)

        # Language independent assets folder
        if assets_folder_url:
            results.append(
                {
                    "title": _(
                        "shared_folder",
                        default="Open ${title} folder",
                        mapping={"title": assets_folder_title},
                    ),
                    "description": _(
                        "description_shared_folder",
                        default="Open the language independent assets folder",
                    ),
                    "action": assets_folder_url,
                    "selected": False,
                    "icon": None,
                    "extra": {
                        "id": "_shared_folder",
                        "separator": results and "actionSeparator" or None,
                        "class": "",
                    },
                    "submenu": None,
                }
            )

        # User preferred language root folder
        if not folder_translatable and not content_translatable:
            results.append(
                {
                    "title": _("language_folder", default="Return to language folder"),
                    "description": _(
                        "description_language_folder",
                        default="Go to the user's browser preferred language "
                        "related folder",
                    ),
                    "action": site_url
                    + "/"
                    + language_tool.getPreferredLanguage(),  # noqa
                    "selected": False,
                    "icon": None,
                    "extra": {
                        "id": "_language_folder",
                        "separator": (
                            (results and not assets_folder_url)
                            and "actionSeparator"
                            or None
                        ),
                        "class": "",
                    },
                    "submenu": None,
                }
            )

        return results


@implementer(ITranslateSubMenuItem)
class TranslateSubMenuItem(BrowserSubMenuItem):
    title = _("label_translate_menu", default="Translate")
    description = _(
        "title_translate_menu", default="Manage translations for your content."
    )
    submenuId = "plone_contentmenu_multilingual"
    order = 5

    @property
    def action(self):
        return self.context.absolute_url() + "/add_translations"

    @property
    def extra(self):
        return {
            "id": "plone-contentmenu-multilingual",
            "li_class": "plonetoolbar-multilingual",
        }

    @view.memoize
    def available(self):
        # Is PAM installed?
        if not IPloneAppMultilingualInstalled.providedBy(self.request):
            return False

        # Do we have portal_languages?
        lt = getToolByName(self.context, "portal_languages", None)
        if lt is None:
            return False

        # Do we have multiple languages?
        supported = lt.listSupportedLanguages()
        if len(supported) < 2:
            return False

        # And now check permissions
        sm = getSecurityManager()
        if not sm.checkPermission(ManageTranslations, self.context):
            return False

        return True

    def selected(self):
        return False



from AccessControl import getSecurityManager
from plone.app.multilingual import _
from plone.app.multilingual.browser.vocabularies import untranslated_languages
from plone.app.multilingual.interfaces import ITranslationManager
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives
from plone.autoform.interfaces import IFormFieldProvider
from plone.base.interfaces import IPloneSiteRoot
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from zope import interface
from zope import schema
from zope.browsermenu.interfaces import IBrowserMenu
from zope.browsermenu.interfaces import IBrowserSubMenuItem
from zope.component.hooks import getSite
from zope.interface import provider
from zope.schema.interfaces import IContextAwareDefaultFactory


def make_relation_root_path(context):
    security_manager = getSecurityManager()
    site = getSite()

    # This should not happen, there's no View permission for the current object
    if not security_manager.checkPermission("View", context):
        return "/".join(site.getPhysicalPath())

    # Try to find the "closest" object in the target language
    current_object = context
    target_language = context.REQUEST.get("language", None)

    if target_language is None:
        return "/".join(site.getPhysicalPath())

    closest_translation = None
    while closest_translation is None:
        current_object = current_object.aq_parent
        translation_manager = ITranslationManager(current_object)
        closest_translation = translation_manager.get_translation(target_language)
        if IPloneSiteRoot.providedBy(closest_translation):
            break

    # If the closest translation has no View permission default to old behavior
    if not security_manager.checkPermission("View", closest_translation):
        return "/".join(site.getPhysicalPath())

    # Everything went well, return the translation's path
    return "/".join(closest_translation.getPhysicalPath())


class IMultilingualLayer(interface.Interface):
    """browser layer"""


class ITranslateSubMenuItem(IBrowserSubMenuItem):
    """The menu item linking to the translate menu."""


class ITranslateMenu(IBrowserMenu):
    """The translate menu."""


class ICreateTranslation(interface.Interface):
    language = schema.Choice(
        title=_("title_language", default="Language"),
        source=untranslated_languages,
    )


class IUpdateLanguage(interface.Interface):
    language = schema.Choice(
        title=_("title_available_languages", default="Available languages"),
        description=_(
            "description_update_language",
            default="Untranslated languages from the current content",
        ),
        source=untranslated_languages,
        required=True,
    )


@provider(IContextAwareDefaultFactory)
def request_language(context):
    return context.REQUEST.form.get("language")


class IConnectTranslation(model.Schema):
    language = schema.Choice(
        title=_("title_language", default="Language"),
        source=untranslated_languages,
        defaultFactory=request_language,
        required=True,
    )
    content = RelationChoice(
        title=_("content"),
        vocabulary="plone.app.multilingual.RootCatalog",
        required=True,
    )
    directives.widget(
        "content",
        RelatedItemsFieldWidget,
        pattern_options={
            "basePath": make_relation_root_path,
        },
    )


interface.alsoProvides(IUpdateLanguage, IFormFieldProvider)
interface.alsoProvides(IConnectTranslation, IFormFieldProvider)



from plone.app.multilingual.browser.setup import SetupMultilingualSite
from plone.app.multilingual.interfaces import IPloneAppMultilingualInstalled
from plone.app.multilingual.testing import PAM_FUNCTIONAL_TESTING
from plone.testing.z2 import Browser
from Products.CMFCore.utils import getToolByName
from zope.interface import alsoProvides

import transaction
import unittest


class TestLanguageSwitcher(unittest.TestCase):
    layer = PAM_FUNCTIONAL_TESTING

    def setUp(self):
        # Set test variables
        self.portal = self.layer["portal"]
        self.language_tool = getToolByName(self.portal, "portal_languages")
        self.portal_url = self.portal.absolute_url()
        self.request = self.layer["request"]
        alsoProvides(self.layer["request"], IPloneAppMultilingualInstalled)

        # Add Indonesian as supported language, because this has corner cases:
        # it's LRF cannot have id "id", so it is "id-id".
        self.language_tool.addSupportedLanguage("id")
        multilingual_setup_tool = SetupMultilingualSite()
        multilingual_setup_tool.setupSite(self.portal)
        transaction.commit()

        # Setup testbrowser
        self.browser = Browser(self.layer["app"])
        self.browser.handleErrors = False

    def test_switcher_redirects_to_default_english(self):
        self.browser.open(self.portal_url)
        self.assertEqual(self.browser.url, self.portal_url + "/en")
        self.assertEqual(self.browser.cookies["I18N_LANGUAGE"], "en")

    def test_switcher_redirects_to_default_indonesian(self):
        self.language_tool.setDefaultLanguage("id")
        transaction.commit()
        self.browser.open(self.portal_url)
        self.assertEqual(self.browser.url, self.portal_url + "/id-id")
        self.assertEqual(self.browser.cookies["I18N_LANGUAGE"], "id")

    def test_switcher_redirects_to_preferred_catalan(self):
        # Tell Plone that we prefer Catalan.
        self.browser.open(self.portal_url + "/ca?set_language=ca")
        # Go to the site root.
        self.browser.open(self.portal_url)
        # We get redirected to our preferred language root folder.
        self.assertEqual(self.browser.url, self.portal_url + "/ca")
        self.assertEqual(self.browser.cookies["I18N_LANGUAGE"], "ca")

    def test_switcher_redirects_to_preferred_indonesian(self):
        # Tell Plone that we prefer Indonesian.
        self.browser.open(self.portal_url + "/id-id?set_language=id")
        # Go to the site root.
        self.browser.open(self.portal_url)
        # We get redirected to our preferred language root folder.
        self.assertEqual(self.browser.url, self.portal_url + "/id-id")
        self.assertEqual(self.browser.cookies["I18N_LANGUAGE"], "id")



from Acquisition import aq_inner
from plone.i18n.interfaces import ILanguageUtility
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from zope.component import getUtility


class LanguageSwitcher(BrowserView):
    def __call__(self):
        """Redirect to the preferred language top-level folder.

        If no folder for preferred language exists, redirect to default
        language.

        Copy from LinguaPlone
        """
        context = aq_inner(self.context)
        plt = getToolByName(context, "portal_languages")
        pref = plt.getPreferredLanguage(self.request)
        default = plt.getDefaultLanguage()
        # Handle Indonesian: its language code "id" is not allowed in Plone as
        # content id, so its LRF is called "id-id".
        pref = "id-id" if pref == "id" else pref
        default = "id-id" if default == "id" else default
        ids = self.context.keys()
        target = (pref in ids) and pref or default
        url = f"{context.absolute_url()}/{target}"

        # We need to set the language cookie on the first response or it will
        # be set on the frontpage itself, making it uncachable
        # In case of Indonesian, we need to use 'id', not 'id-id'.
        target = "id" if target == "id-id" else target
        tool = getUtility(ILanguageUtility)
        # setLanguageCookie calls getLanguageCookie, and only sets a cookie when needed.
        tool.setLanguageCookie(target, request=self.request)

        self.request.response.redirect(url, status=302)



from Acquisition import aq_base
from plone.app.multilingual.browser.setup import SetupMultilingualSite
from plone.app.multilingual.browser.vocabularies import AllContentLanguageVocabulary
from plone.app.multilingual.interfaces import ATTRIBUTE_NAME
from plone.app.multilingual.interfaces import IPloneAppMultilingualInstalled
from plone.app.multilingual.testing import PAM_INTEGRATION_PRESET_TESTING
from plone.app.multilingual.testing import PAM_INTEGRATION_TESTING
from plone.base.interfaces import ILanguage
from Products.CMFCore.utils import getToolByName
from zope.interface import alsoProvides

import unittest


class TestSetupMultilingualSite(unittest.TestCase):
    """Testing multilingual site without predefined languages."""

    layer = PAM_INTEGRATION_TESTING

    def setUp(self):
        """Setting up the test."""
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.language_tool = getToolByName(self.portal, "portal_languages")
        self.languages = self.language_tool.getSupportedLanguages()
        alsoProvides(self.layer["request"], IPloneAppMultilingualInstalled)

    def test_single_language(self):
        """Only one language is set."""
        self.assertEqual(len(self.languages), 1)

    def test_portal_has_tg_attribute(self):
        """The site root should have the TG attribute set after installing"""
        tg_attribute = getattr(aq_base(self.portal), ATTRIBUTE_NAME, None)
        self.assertIsNotNone(tg_attribute)

    def test_no_languagefolder_created(self):
        """On a single language no folder creation is done."""
        portal_objects = self.portal.objectIds()
        for lang in self.languages:
            self.assertNotIn(lang, portal_objects)

    def test_all_supported_languages(self):
        """There is a language which code is 'id'.

        This is Indonesian, or Bahasa Indonesia in that language.
        This broke the root language folder setup process.
        To get rid of that the folder is 'id-id'.
        Here we test all languages, to be sure we catch such a corner case.
        """
        all_langs = AllContentLanguageVocabulary()(self.portal)
        for lang in all_langs:
            self.language_tool.addSupportedLanguage(lang.value)

        workflow_tool = getToolByName(self.portal, "portal_workflow")
        workflow_tool.setDefaultChain("simple_publication_workflow")

        setup_tool = SetupMultilingualSite()
        setup_tool.setupSite(self.portal)

        portal_objects = self.portal.objectIds()

        for lang in all_langs.by_value.keys():
            # Special handling for Indonesian.
            folder_id = "id-id" if lang == "id" else lang
            self.assertIn(folder_id, portal_objects)
            folder = self.portal[folder_id]
            self.assertEqual(ILanguage(folder).get_language(), lang)

    def test_type_of_language_folders(self):
        """The created objects have to be 'Language Root Folder'."""
        all_langs = AllContentLanguageVocabulary()(self.portal)
        for lang in all_langs:
            self.language_tool.addSupportedLanguage(lang.value)

        workflow_tool = getToolByName(self.portal, "portal_workflow")
        workflow_tool.setDefaultChain("simple_publication_workflow")

        setup_tool = SetupMultilingualSite()
        setup_tool.setupSite(self.portal)

        for lang in all_langs.by_value.keys():
            # Special handling for Indonesian.
            folder_id = "id-id" if lang == "id" else lang
            folder = self.portal[folder_id]
            self.assertEqual(folder.portal_type, "LRF")


class TestSetupMultilingualPresetSite(unittest.TestCase):
    """Testing multilingual site with predefined languages."""

    layer = PAM_INTEGRATION_PRESET_TESTING

    def setUp(self):
        """Setting up the test."""
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.language_tool = getToolByName(self.portal, "portal_languages")
        self.languages = self.language_tool.getSupportedLanguages()
        alsoProvides(self.layer["request"], IPloneAppMultilingualInstalled)

    def test_language_folders_created(self):
        """Available languages are: 'en', 'ca', 'es'.
        After setup has run there should be
        new objects in the portal named after the languages.
        """
        portal_objects = self.portal.objectIds()
        for lang in self.languages:
            self.assertIn(lang, portal_objects)

    def test_type_of_language_folders(self):
        """The created objects have to be 'Language Root Folder'."""
        for lang in self.languages:
            self.assertEqual(self.portal.get(lang).portal_type, "LRF")



"""Setup plone.app.multilingual."""
from setuptools import find_packages
from setuptools import setup


setup(
    # metadata in setup.cfg
    packages=find_packages("src"),
    package_dir={"": "src"},
    namespace_packages=["plone", "plone.app"],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "plone.i18n>=4.0.4",
        "setuptools",
        "BTrees",
        "Products.CMFCore",
        "Products.CMFPlone",
        "Products.GenericSetup",
        "Products.statusmessages",
        "plone.app.content",
        "plone.app.contentmenu",
        "plone.app.dexterity",
        "plone.app.event",
        "plone.app.i18n",
        "plone.app.layout",
        "plone.app.registry",
        "plone.app.uuid",
        "plone.app.vocabularies",
        "plone.app.z3cform",
        "plone.autoform",
        "plone.base",
        "plone.behavior",
        "plone.dexterity",
        "plone.indexer",
        "plone.locking",
        "plone.memoize",
        "plone.protect",
        "plone.registry",
        "plone.schemaeditor",
        "plone.supermodel",
        "plone.uuid",
        "plone.z3cform",
        "z3c.form",
        "z3c.relationfield",
        "zc.relation",
        "zope.browsermenu",
        "zope.intid",
        "zope.pagetemplate",
    ],
    extras_require={
        "test": [
            "plone.app.testing[robot]",
            "plone.app.robotframework",
            "lxml",
            "plone.api",
            "plone.app.contenttypes",
            "plone.app.relationfield",
            "plone.browserlayer",
            "plone.rfc822",
            "plone.testing",
            "robotsuite",
        ],
    },
    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """,
)
